package net.zzforrest.ld29;

import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.Graphics;
import java.io.IOException;
import java.util.ArrayList;

import com.sun.glass.events.KeyEvent;

import net.zzforrest.ld29.animation.Animation;
import net.zzforrest.ld29.entity.*;
import net.zzforrest.ld29.menu.*;

public class Game
{
	private Animation heart = new Animation(Art.heart, 0, 0, 40, 40, 1, 1);

	private ArrayList<Entity> entities;
	private Player player;
	private Pin[] pins;
	private Block[][] blocks;
	private Bitmap level;

	private Menu menu;
	private int screens;
	private int score;
	private int score_d;

	private Font font;

	public Game()
	{
		menu = new MainMenu(this);

		entities = new ArrayList<Entity>();
		player = new Player(this);
		pins = new Pin[4];
		entities.add(player);
		blocks = new Block[32][24];

		level = new Bitmap("levels.png");

		try
		{
			font = Font.createFont(Font.TRUETYPE_FONT,
					this.getClass().getResourceAsStream("/fonts/ADayinSeptember.otf"))
					.deriveFont(20f);
		} catch (FontFormatException e)
		{
			e.printStackTrace();
		} catch (IOException e)
		{
			e.printStackTrace();
		}

		startLevel();
	}

	public void input(Input input)
	{
		if(input.getKeyPressed(KeyEvent.VK_M))
			Art.mute();
		
		if (menu != null)
		{
			menu.input(input);
			return;
		}
		if(input.getKeyPressed(KeyEvent.VK_P) || !input.focused)
			pause();
		
		player.input(input);
		if(screens < 6 && input.getKeyPressed(KeyEvent.VK_ENTER))
		{
			screens = 6;
			startLevel();
			menu = new Transition(this, new Blank(this, true), new Blank(this, false));
		}
	}

	public void update()
	{
		if (menu != null)
		{
			menu.update();
			return;
		}

		for (int i = 0; i < entities.size(); i++)
		{
			entities.get(i).update();
			if (entities.get(i).isdead())
			{
				entities.remove(i--);
			}
		}

		if (score_d < score)
			score_d += 5;

		// win condition
		boolean won = true;
		for (int i = 0; i < pins.length; i++)
		{
			if (!pins[i].isdead())
				won = false;
		}

		if (won)
		{
			screens++;
			score(600);
			startLevel();
			menu = new Transition(this, new Blank(this, true), new Blank(this, false));
		}

		// lose condition
		if (player.getState() == Player.State.DEAD
				&& player.getAnimation().getState() == Animation.State.ENDED)
		{
			menu = new Transition(this, new Blank(this, true), new Lose(this));
		}
	}

	public void render(Graphics g)
	{
		g.setFont(font);
		
		g.drawImage(Art.background, 0, 0, null);
		
		if (menu != null)
		{
			menu.render(g, 80, 60);
			return;
		}

		renderGame(g);
	}

	public void renderGame(Graphics g)
	{
		Graphics g2 = g;
		g2.translate(80, 60);

		g2.drawImage(Art.paper, 0, 0, null);

		for (int i = 0; i < 32; i++)
		{
			for (int j = 0; j < 24; j++)
			{
				blocks[i][j].render(g2, i * 20, j * 20);
			}
		}

		for (Entity entity : entities)
			entity.render(g2);

		for (int i = 0; i < player.getHealth(); i++)
		{
			heart.render(g2, 300 + 50 * i, -50);
		}

		g.drawString("Score : " + score_d, 50, -10);
		
		if(screens < 6)
		{
			switch(screens)
			{
			case 0:
				g.drawString("'Is there something beneath the surface?'", 60, 50);
				g.drawString("wondered the young drawing.", 60, 100);
				g.drawString("'I want to find out!'", 60, 150);
				g.drawString("Move with Arrow Keys", 60, 250);
				g.drawString("Pull the pins with X or Space", 60, 300);
				g.drawString("Skip tutorial with Enter", 60, 350);
				break;
			case 1:
				g.drawString("There are many obstacles that block his descent.", 60, 50);
				break;
			case 2:
				g.drawString("However, he was mobile for a sketch.", 80, 50);
				break;
			case 3:
				g.drawString("And cautious.", 60, 50);
				break;
			case 4:
				g.drawString("The forgotten drawings, deep in the stack,", 100, 50);
				g.drawString("were unhappy to see him.", 140, 100);
				break;
			case 5:
				g.drawString("They set up traps to end his journey short.", 100, 50);
				g.drawString("He knew he wasn't the first.", 100, 100);
				g.drawString("He knew he won't be the last.", 100, 150);
				break;
			}
		}
	}

	public void setMenu(Menu menu)
	{
		this.menu = menu;
	}

	public void startLevel()
	{
		if(screens == 6)
		{
			score = 0;
			score_d = 0;
			player.reset();
		}
		
		entities.clear();

		player.spawn();
		entities.add(player);
		pins[0] = new Pin(this, 0, 0);
		pins[1] = new Pin(this, 600, 0);
		pins[2] = new Pin(this, 0, 440);
		pins[3] = new Pin(this, 600, 440);

		int lx = 0, ly = 0;

		if (screens < 6)
		{
			lx = 32 * screens;
		} else if (screens < 13)
		{
			ly = 24;
			lx = 32 * Util.random.nextInt(8);
		} else if (screens == 13)
		{
			lx = 256;
		} else if (screens < 25)
		{
			lx = 32 * Util.random.nextInt(10);
			ly = 48;
		} else if(screens == 25)
		{
			lx = 288;
		} else if(screens < 40)
		{
			lx = 32 * Util.random.nextInt(15);
			ly = 72;
		} else if (screens == 40)
		{
			lx = 320;
		} else
		{
			if(screens % 25 == 0)
				lx = 320;
			else
			{
				lx = 32 * Util.random.nextInt(20);
				ly = 96;
			}
		}

		for (int i = 0; i < 32; i++)
		{
			for (int j = 0; j < 24; j++)
			{
				int col = level.getPixel(lx + i, ly + j) & 0xffffff;
				
				blocks[i][j] = new Block(col);
				
				int tx = i * 20, ty = j * 20;
				
				switch (col)
				{
				case 0x00ffff:
					entities.add(new Roamer(this, tx, ty));
					break;
				case 0x01ffff:
					entities.add(new Spike(this, tx, ty));
					break;
				case 0x02ffff:
					entities.add(new MovingSpike(this, tx, ty));
					break;
				case 0x03ffff:
					entities.add(new Coin(this, tx, ty));
					break;
				case 0x04ffff:
					entities.add(new Heart(this, tx, ty));
					break;
				case 0x05ffff:
					entities.add(new Shooter(this, tx, ty));
				default:
					break;
				}
			}
		}

		for (int i = 0; i < pins.length; i++)
			entities.add(pins[i]);
	}

	public Player getPlayer()
	{
		return player;
	}

	public Block[][] getBlocks()
	{
		return blocks;
	}

	public void score(int score)
	{
		this.score += score;
	}

	public int getScore()
	{
		return score;
	}
	
	public void reset()
	{
		entities.clear();
		player = new Player(this);
		score = 0;
		score_d = 0;
		screens = 0;
		startLevel();
	}
	
	public void pause()
	{
		menu = new Pause(this);
	}
	
	public void spawnEntity(Entity e)
	{
		entities.add(e);
	}
}