package net.zzforrest.ld29;

import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class Main extends JPanel implements Runnable
{
	private static final long serialVersionUID = 1L;
	
	public static final int width = 800;
	public static final int height = 600;
	public static final long fps = 60;
	public static final String title = "Under the Paper by zzforrest";
	
	private Thread thread;
	private boolean running;
	
	private Input input;
	private Game game;
	
	public Main()
	{
		setPreferredSize(new Dimension(width, height));
		
		game = new Game();
		
		input = new Input();
		addKeyListener(input);
		addMouseListener(input);
		addMouseMotionListener(input);
		addFocusListener(input);
		
		setFocusable(true);
		requestFocus();
	}
	
	public void start()
	{
		if(running)
			return;
		running = true;
		thread = new Thread(this);
		thread.start();
	}
	
	public void run()
	{
		int frames = 0;
		long lastTime = System.currentTimeMillis();
		
		while(running)
		{
			game.input(input);
			input.update();
			game.update();
			repaint();
			
			try
			{
				Thread.sleep(1000 / fps);
			}
			catch (InterruptedException e)
			{
				e.printStackTrace();
			}
			
			frames ++;
			long now = System.currentTimeMillis();
			if(now - lastTime >= 1000)
			{
				System.out.println(frames);
				frames = 0;
				lastTime += 1000;
			}
		}
		
		stop();
	}
	
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		game.render(g);
	}
	
	public void stop()
	{
		if(!running)
			return;
		running = false;
		try
		{
			thread.join();
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args)
	{
		Main main = new Main();
		JFrame frame = new JFrame(title);
		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setResizable(false);
		frame.add(main);
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
		
		main.start();
	}
}