package net.zzforrest.ld29.animation;

import java.awt.Graphics;
import java.awt.image.BufferedImage;

public class Animation
{
	public enum State
	{
		PAUSED, PLAYING, ENDED, LOOPING
	};
	
	BufferedImage source;
	int x, y, w, h;
	int xframes, yframes;
	int framewidth, frameheight;
	int delay;
	
	int frame;
	int loops;
	
	State state;
	
	int tick;
	
	public Animation(BufferedImage source, int x, int y, int w, int h, int xframes, int yframes)
	{
		this.source = source;
		this.x = x;
		this.y = y;
		this.w = w;
		this.h = h;
		this.xframes = xframes;
		this.yframes = yframes;
		
		framewidth = w / xframes;
		frameheight = h / yframes;
		
		delay = 1;
		
		state = State.PAUSED;
	}
	
	public void update()
	{
		tick ++;
		
		switch(state)
		{
		case PLAYING:
			if(tick % delay == 0)
				frame ++;
			if(frame == xframes * yframes)
			{
				frame --;
				state = State.ENDED;
			}
			break;
		
		case PAUSED:
			break;
		
		case LOOPING:
			if(tick % delay == 0)
				frame ++;
			if(frame == xframes * yframes)
			{
				loops --;
				frame = 0;
				if(loops == 0)
					state = State.ENDED;
			}
			break;
		
		case ENDED:
			break;
		
		default:
			break;
		}
	}
	
	public void render(Graphics g, int dx, int dy)
	{
		int sx = x + (frame % xframes) * framewidth, sy = y + (frame / xframes) * frameheight;
		g.drawImage(source, dx, dy, dx + framewidth, dy + frameheight,
				sx, sy, sx + framewidth, sy + frameheight, null);
	}
	
	public void play()
	{
		state = State.PLAYING;
	}
	
	public void pause()
	{
		state = State.PAUSED;
	}
	
	public Animation loop(int loops)
	{
		state = State.LOOPING;
		this.loops = loops;
		return this;
	}
	
	public void stop()
	{
		state = State.ENDED;
		frame = 0;
	}
	
	public State getState()
	{
		return state;
	}
	
	public Animation setDelay(int delay)
	{
		this.delay = delay;
		return this;
	}
	
	public Animation setFrame(int frame)
	{
		this.frame = frame;
		return this;
	}
	public int getFrame()
	{
		return frame;
	}
	
	public int getFrameW()
	{
		return framewidth;
	}
	
	public int getFrameH()
	{
		return frameheight;
	}
}