package net.zzforrest.ld29.entity;

import java.awt.Graphics;

import net.zzforrest.ld29.Art;
import net.zzforrest.ld29.Game;
import net.zzforrest.ld29.Util;
import net.zzforrest.ld29.animation.Animation;

public class Pin extends Entity
{
	Animation animation;

	public enum State
	{
		PINNED, FALLING
	};

	State state;
	int tick;

	private float xvel;
	private float yvel;
	private float gravity = 0.6f;

	public Pin(Game game, float x, float y)
	{
		super(game);
		this.x = x;
		this.y = y;

		w = 40;
		h = 40;

		animation = new Animation(Art.pin, 0, 0, 80, 57, 2, 1);
		state = State.PINNED;
	}

	@Override
	public void update()
	{
		if (state == State.FALLING)
		{
			x += xvel;
			y += (yvel += gravity);
			if (y > 800)
				die();

			tick++;
		}

		animation.update();

		if (Util.isColliding(this, game.getPlayer())
				&& game.getPlayer().isInteracting())
		{
			if(state == State.PINNED)
				game.score(100);
			else
				game.score(5);
			
			state = State.FALLING;
			animation.setFrame(1);
			xvel = (Util.random.nextFloat() - .5f) * 6;
			yvel = -Util.random.nextFloat() * 10;
			
			Art.playClip(Art.pindrop);
		}
	}

	@Override
	public void render(Graphics g)
	{
		switch (state)
		{
		case PINNED:
			animation.render(g, (int) x, (int) y);
			break;
		case FALLING:
			Util.renderRotatedAnimation(animation, (int) x, (int) y, (int) x
					+ w / 2, (int) y + h / 2, tick / 5.0, g);
			break;
		default:
			break;
		}
	}

	public State getState()
	{
		return state;
	}
}