package net.zzforrest.ld29.entity;

import java.awt.Graphics;
import java.awt.event.KeyEvent;

import net.zzforrest.ld29.Art;
import net.zzforrest.ld29.Block;
import net.zzforrest.ld29.Game;
import net.zzforrest.ld29.Input;
import net.zzforrest.ld29.animation.Animation;

public class Player extends Entity
{
	public enum State
	{
		NORMAL, HURT, DEAD
	};

	public static final int DELAY = 5;
	final Animation WALK_DOWN = new Animation(Art.player, 0, 0, 200, 35, 8, 1)
			.setDelay(DELAY);
	final Animation WALK_LEFT = new Animation(Art.player, 0, 35, 100, 35, 4, 1)
			.setDelay(DELAY);
	final Animation WALK_RIGHT = new Animation(Art.player, 100, 35, 100, 35, 4,
			1).setDelay(DELAY);
	final Animation WALK_UP = new Animation(Art.player, 0, 70, 200, 35, 8, 1)
			.setDelay(DELAY);
	final Animation DIE = new Animation(Art.player, 0, 105, 200, 70, 8, 2)
			.setDelay(DELAY);

	Animation animation;

	private float acceleration = 0.77f;
	private float friction = 0.75f;

	private int health;
	private boolean interacting;

	private State state;
	int hurttick;
	int tick;

	public Player(Game game)
	{
		super(game);

		animation = WALK_DOWN;
		x = 320;
		y = 240;
		w = 20;
		h = 30;

		health = 3;

		state = State.NORMAL;
	}

	public void input(Input input)
	{
		if (state == State.DEAD)
			return;

		int ix = 0, iy = 0; // impulses

		if (input.getKey(KeyEvent.VK_LEFT))
			ix--;
		if (input.getKey(KeyEvent.VK_RIGHT))
			ix++;
		if (input.getKey(KeyEvent.VK_UP))
			iy--;
		if (input.getKey(KeyEvent.VK_DOWN))
			iy++;

		if (input.getKeyPressed(KeyEvent.VK_X)
				|| input.getKeyPressed(KeyEvent.VK_SPACE))
			interacting = true;
		else
			interacting = false;

		xvel += ix * acceleration;
		yvel += iy * acceleration;

		if (iy == -1)
			animation = WALK_UP.loop(-1);
		else if (iy == 1)
			animation = WALK_DOWN.loop(-1);
		else if (ix == -1)
			animation = WALK_LEFT.loop(-1);
		else if (ix == 1)
			animation = WALK_RIGHT.loop(-1);
		else
			animation.stop();
	}

	public void update()
	{
		Block[][] blocks = game.getBlocks();

		for (int i = (int) Math.floor(x + xvel) / 20; i < Math.floor(x + w
				+ xvel) / 20; i++)
		{
			if (i < 0 || i >= 32)
				break;
			for (int j = (int) Math.floor(y) / 20; j < Math.floor(y + h) / 20; j++)
			{
				if (j < 0 || j >= 24)
					break;

				if (blocks[i][j].isSolid())
				{
					xvel = 0; // if it's in range, it is impossible for it not
								// to collide. yay technicalities
				}
			}
		}

		x += xvel;

		for (int i = (int) Math.floor(x) / 20; i < Math.floor(x + w) / 20; i++)
		{
			if (i < 0 || i >= 32)
				break;
			for (int j = (int) Math.floor(y + yvel) / 20; j < Math.floor(y + h
					+ yvel) / 20; j++)
			{
				if (j < 0 || j >= 24)
					break;

				if (blocks[i][j].isSolid())
				{
					yvel = 0;
				}
			}
		}

		y += yvel;

		xvel *= friction;
		yvel *= friction;

		if (x < 0)
			x = 0;
		if (x + w > 640)
			x = 640 - w;
		if (y < 0)
			y = 0;
		if (y + h > 480)
			y = 480 - h;

		animation.update();

		tick++;

		if (state == State.HURT && tick - hurttick == 60)
		{
			state = State.NORMAL;
		}
	}

	public void render(Graphics g)
	{
		if (state == State.HURT && tick / 2 % 2 == 0)
			return;

		animation.render(g, (int) x + w / 2 - animation.getFrameW() / 2,
				(int) y + h - animation.getFrameH());
	}

	public int getHealth()
	{
		return health;
	}

	public void spawn()
	{
		x = 320;
		y = 240;

		xvel = 0;
		yvel = 0;
	}

	public boolean isInteracting()
	{
		return interacting;
	}

	public void hurt()
	{
		if (state == State.HURT || state == State.DEAD)
			return;
		
		Art.playClip(Art.hurt);
		
		state = State.HURT;
		hurttick = tick;

		health--;
		if (health == 0)
		{
			Art.playClip(Art.die);
			
			state = State.DEAD;
			animation = DIE;
			animation.play();

			xvel = 0;
			yvel = 0;
		}
	}

	public Animation getAnimation()
	{
		return animation;
	}

	public State getState()
	{
		return state;
	}

	public void reset()
	{
		animation = WALK_DOWN;
		x = 320;
		y = 240;
		w = 20;
		h = 30;

		health = 3;

		state = State.NORMAL;
	}
	
	public void heal(int health)
	{
		this.health += health;
	}
}